// C Source File
// Created 11/21/2011; 11:53:55 AM

// Comments by TC01
// 68k C program that parses and displays the #define COMMENT_... lines directly below this

#define COMMENT_STRING         "Access program metadata in assembly programs using dataext"
#define COMMENT_PROGRAM_NAME   "Comments"
#define COMMENT_VERSION_NUMBER 0,2,0,0
#define COMMENT_AUTHORS        "TC01"

#include <alloc.h>
#include <args.h>
#include <dialogs.h>
#include <estack.h>
#include <error.h>
#include <kbd.h>
#include <stdio.h>
#include <string.h>
#include <vat.h>

#include "dataext.h"

void _main(void)
{
	FILE* program;
	SYM_ENTRY* symData;
	DATA_EXT* programInfo = NULL;
	ESI argptr;
	ESI filetype;
	
	char* filename = NULL;
	char* working = NULL;
	char* prgmName = NULL;
	char* prgmVersion = NULL;
	char* prgmAuthors = NULL;
	char* prgmComment = NULL;
	char* programData = NULL;
	int fileSize;
	int argtype;
	
	//Argument parsing for command line input
	filename = calloc(20, sizeof(char));
	if (ArgCount() == 1)
	{
		InitArgPtr(argptr);
		while ((argtype = GetArgType(argptr)) != END_TAG)
		{
			if (argtype == STR_TAG)
				strcpy(filename, GetStrnArg(argptr));
			else
				ER_throw(ER_ARGUMENT);
		}
	}
	else
		ER_throw(ER_ARGUMENT);
	
	//See if the file exists
	symData = SymFindPtr(SYMSTR(filename), 0);
	if (symData == NULL)
	{
		DlgMessage("Comments", "Error: File does not exist", BT_OK, BT_NONE);
		free(filename);
		return;
	}
	
	//Check what sort of file it is
	fileSize = ((MULTI_EXPR*)HeapDeref(symData->handle))->Size + 2;
	filetype = HToESI(symData->handle);
	if (*filetype != ASM_TAG)
	{
		DlgMessage("Comments", "Error: File is not an assembly program", BT_OK, BT_NONE);
		free(filename);
		return;		
	}
	
	//Now attempt to open the file
	program = fopen(filename, "rb");
	if (program == NULL)
	{
		DlgMessage("Comments", "Error: Unable to open file", BT_OK, BT_NONE);
		free(filename);
		return;
	}
	
	//Get a data extension pointer to the program
	programData = calloc(fileSize, sizeof(char));
	fread(programData, fileSize, sizeof(char), program);
	programInfo = DataExtensionPointer(programData);
	if (programInfo == NULL)
	{
		DlgMessage("Comments", "Error: Unable to access file metadata", BT_OK, BT_NONE);
		free(filename);
		fclose(program);
		return;
	}
	
	//Get relevant data from the data extension pointer
	prgmName = GetDEProgramNameEx(programInfo, "");
	prgmAuthors = GetDEAuthorsEx(programInfo, "");
	prgmVersion = GetDEVersionString(programInfo);
	prgmComment = GetDECommentEx(programInfo, "");
	
	//Output the data in a nice message box
	working = calloc(250, sizeof(char));
	sprintf(working, "File Name: %s\nName: %s\nVersion: %s\nAuthors: %s\n\n%s", filename, prgmName, prgmVersion, prgmAuthors, prgmComment);
	DlgMessage("Comments", working, BT_OK, BT_NONE);
	
	//Free all the memory
	fclose(program);
	free(prgmVersion);
	free(filename);
	free(working);
}
