// STANDARD FOR NOSTUB COMMENT AND OTHER EXPORTED STATIC DATA EXTENSIONS
// Reference implementation version 0.92 beta
// Implemented specification version: 1.0

// Data extension types
typedef unsigned short DATA_EXT_TYPE;
#define DE_COMMENT ((unsigned short)0u)
#define DE_PROGRAM_NAME ((unsigned short)1u)
#define DE_VERSION_STRING ((unsigned short)2u)
#define DE_VERSION_NUMBER ((unsigned short)3u)
#define DE_BW_ICON ((unsigned short)4u)
#define DE_GRAYSCALE_ICON ((unsigned short)5u)
#define DE_INCOMPAT_FLAGS ((unsigned short)6u)
#define DE_AUTHORS ((unsigned short)7u)

// Numeric version number, used in the extension header and for extension type
// DE_VERSION_NUMBER.
typedef union VERSION_NUMBER {
  unsigned long version_number;
  struct {
    unsigned char major;
    unsigned char minor;
    unsigned char revision;
    unsigned char subrev;
  } version_struct;
  struct {
    unsigned char major;
    unsigned char minor;
    unsigned char revision;
    unsigned char subrev;
  };
} VERSION_NUMBER;

// Incompatibility flag bitfield.
typedef union INCOMPAT_FLAGS {
  unsigned long all_flags;
  struct {
    unsigned long reserved:27, needs_all_stack:1, uses_ev_hook:1,
                  uses_vectors:1, uses_traps:1, creates_handles:1;
  };
} INCOMPAT_FLAGS;

// Entry for an extension in the extension header.
typedef struct EXTENSION_ENTRY {
  DATA_EXT_TYPE type_of_extension;
  unsigned short offset_of_extension;
} EXTENSION_ENTRY;

// Structure representing a data extension header.
typedef struct DATA_EXT {
  VERSION_NUMBER standard_revision;
  unsigned short num_extensions;
  EXTENSION_ENTRY extensions[];
} DATA_EXT;

// Functions. Some of those could be implemented as macros, but they aren't
// on purpose, in order to allow assembly programs to use the complete API.

/* Takes a pointer to the contents of a file (with the size bytes already
   skipped). Checks if the file contains a data extension header. Returns a
   pointer to the data extension header if there is one, or NULL otherwise. */
DATA_EXT *DataExtensionPointer(void *fptr asm("a0"));

/* Takes a pointer to a data extension header. Computes the pointer to the file
   contents from the pointer to the data extension header, assuming that the
   data extension header is left in place, not copied. Also takes the requested
   extension type. Returns a pointer to the requested extension if it is
   present, or NULL otherwise. */
void *SearchDataExtension(DATA_EXT *extptr asm("a0"),
                          DATA_EXT_TYPE exttype asm("d0"));

/* Takes a pointer to a data extension header and a pointer to the contents of
   a file (with the size bytes already skipped) and the requested extension
   type. Returns a pointer to the requested extension if it is present, or NULL
   otherwise. */
void *SearchDataExtensionEx(DATA_EXT *extptr asm("a0"), void *fptr asm("a1"),
                            DATA_EXT_TYPE exttype asm("d0"));

/* Takes a pointer to a data extension header. Computes the pointer to the file
   contents from the pointer to the data extension header, assuming that the
   data extension header is left in place, not copied. Returns a pointer to the
   comment if it is present (possibly falling back to the program name), or NULL
   otherwise. */
const char *GetDEComment(DATA_EXT *extptr asm("a0"));

/* Takes a pointer to a data extension header. Computes the pointer to the file
   contents from the pointer to the data extension header, assuming that the
   data extension header is left in place, not copied. Also takes a (possibly
   NULL) pointer to a default comment. Returns a pointer to the comment if it
   is present (possibly falling back to the program name), or the default
   comment otherwise. */
const char *GetDECommentEx(DATA_EXT *extptr asm("a0"),
                           const char *default_comment asm("a1"));

/* Takes a pointer to a data extension header. Computes the pointer to the file
   contents from the pointer to the data extension header, assuming that the
   data extension header is left in place, not copied. Returns a pointer to the
   program name if it is present, or NULL otherwise. */
const char *GetDEProgramName(DATA_EXT *extptr asm("a0"));

/* Takes a pointer to a data extension header. Computes the pointer to the file
   contents from the pointer to the data extension header, assuming that the
   data extension header is left in place, not copied. Also takes a (possibly
   NULL) pointer to a default program name. Returns a pointer to the program
   name if it is present, or the default program name otherwise. */
const char *GetDEProgramNameEx(DATA_EXT *extptr asm("a0"),
                               const char *default_program_name asm("a1"));

/* Takes a pointer to a data extension header. Computes the pointer to the file
   contents from the pointer to the data extension header, assuming that the
   data extension header is left in place, not copied. Returns a handle
   allocated with HeapAllocPtr (malloc) containing a version string, either
   from a DE_VERSION_STRING entry or a formatted version of a DE_VERSION_NUMBER
   entry. Returns NULL if there was no version number or if there was not
   enough memory to allocate the handle. Will cause heap compression. */
char *GetDEVersionString(DATA_EXT *extptr asm("a0"));

/* Takes a pointer to a data extension header. Computes the pointer to the file
   contents from the pointer to the data extension header, assuming that the
   data extension header is left in place, not copied. Returns a handle
   allocated with HeapAllocPtr (malloc) containing a version string, either
   from a DE_VERSION_STRING entry or a formatted version of a DE_VERSION_NUMBER
   entry. Returns NULL if there was no version number. Throws a memory error if
   there was not enough memory to allocate the handle. Will cause heap
   compression. */
char *GetDEVersionStringThrow(DATA_EXT *extptr asm("a0"));

/* Takes a pointer to a data extension header. Computes the pointer to the file
   contents from the pointer to the data extension header, assuming that the
   data extension header is left in place, not copied. Returns a handle
   allocated with HeapAllocPtr (malloc) containing a version string, either
   from a DE_VERSION_STRING entry or a formatted version of a DE_VERSION_NUMBER
   entry. If both are present, shows the DE_VERSION_STRING and the formatted
   DE_VERSION_NUMBER in parentheses. Returns NULL if there was no version number
   or if there was not enough memory to allocate the handle. Will cause heap
   compression. */
char *GetDEVersionStringAndNumber(DATA_EXT *extptr asm("a0"));

/* Takes a pointer to a data extension header. Computes the pointer to the file
   contents from the pointer to the data extension header, assuming that the
   data extension header is left in place, not copied. Returns a handle
   allocated with HeapAllocPtr (malloc) containing a version string, either
   from a DE_VERSION_STRING entry or a formatted version of a DE_VERSION_NUMBER
   entry. If both are present, shows the DE_VERSION_STRING and the formatted
   DE_VERSION_NUMBER in parentheses. Returns NULL if there was no version
   number. Throws a memory error if there was not enough memory to allocate the
   handle. Will cause heap compression. */
char *GetDEVersionStringAndNumberThrow(DATA_EXT *extptr asm("a0"));

/* Takes a pointer to a data extension header. Computes the pointer to the file
   contents from the pointer to the data extension header, assuming that the
   data extension header is left in place, not copied. Returns a pointer
   to a VERSION_NUMBER union if it is present, or NULL otherwise. */
VERSION_NUMBER *GetDEVersionNumber(DATA_EXT *extptr asm("a0"));

/* Takes a pointer to a data extension header. Computes the pointer to the file
   contents from the pointer to the data extension header, assuming that the
   data extension header is left in place, not copied. Also takes the x and y
   coordinates of the destination position. If an icon is present, ORs it to the
   indicated plane using DrawIcon from AMS. Returns TRUE if an icon was drawn,
   or FALSE if there was none. */
short DrawDEBWIcon(DATA_EXT *extptr asm("a0"), void *plane asm("a1"),
                   unsigned short x asm("d0"), unsigned short y asm("d1"));

/* Takes a pointer to a data extension header. Computes the pointer to the file
   contents from the pointer to the data extension header, assuming that the
   data extension header is left in place, not copied. Also takes the x and y
   coordinates of the destination position. If an icon is present, ORs it to the
   indicated planes using DrawIcon from AMS. Returns TRUE if an icon was drawn,
   or FALSE if there was none. */
short DrawDEGrayscaleIcon(DATA_EXT *extptr asm("a0"), void *dplane asm("a1"),
                          void *lplane asm("a2"), unsigned short x asm("d0"),
                          unsigned short y asm("d1"));

/* Takes a pointer to a data extension header. Computes the pointer to the file
   contents from the pointer to the data extension header, assuming that the
   data extension header is left in place, not copied. Also takes the x and y
   coordinates of the destination position and the destination horizontal and
   vertical size. If an icon is present, ORs it to the indicated plane using
   the ExtGraph sprite scaling routines. Requires extgraph.a. Returns TRUE if
   an icon was drawn, or FALSE if there was none. */
short DrawDEScaledBWIcon(DATA_EXT *extptr asm("a0"), void *plane asm("a1"),
                         unsigned short x asm("d0"), unsigned short y asm("d1"),
                         unsigned short hsize asm("d2"),
                         unsigned short vsize asm("d3"));

/* Takes a pointer to a data extension header. Computes the pointer to the file
   contents from the pointer to the data extension header, assuming that the
   data extension header is left in place, not copied. Also takes the x and y
   coordinates of the destination position and the destination horizontal and
   vertical size. If an icon is present, ORs it to the indicated planes using
   the ExtGraph sprite scaling routines by Requires extgraph.a. Returns TRUE if
   an icon was drawn, or FALSE if there was none. */
short DrawDEScaledGrayscaleIcon(DATA_EXT *extptr asm("a0"),
                                void *dplane asm("a1"), void *lplane asm("a2"),
                                unsigned short x asm("d0"),
                                unsigned short y asm("d1"),
                                unsigned short hsize asm("d2"),
                                unsigned short vsize asm("d3"));

/* Takes a pointer to a data extension header. Computes the pointer to the file
   contents from the pointer to the data extension header, assuming that the
   data extension header is left in place, not copied. Returns an
   INCOMPAT_FLAGS union if it is present, or 0 (no incompatibilities) otherwise.
   */
INCOMPAT_FLAGS GetDEIncompatFlags(DATA_EXT *extptr asm("a0"));

/* Takes a pointer to a data extension header. Computes the pointer to the file
   contents from the pointer to the data extension header, assuming that the
   data extension header is left in place, not copied. Returns a pointer to the
   author name(s) if the entry is present, or NULL otherwise. */
const char *GetDEAuthors(DATA_EXT *extptr asm("a0"));

/* Takes a pointer to a data extension header. Computes the pointer to the file
   contents from the pointer to the data extension header, assuming that the
   data extension header is left in place, not copied. Also takes a (possibly
   NULL) pointer to a default program name. Returns a pointer to the author
   name(s) if the entry is present, or the default author name otherwise. */
const char *GetDEAuthorsEx(DATA_EXT *extptr asm("a0"),
                           const char *default_author_name asm("a1"));
